#include "jpegdecoder.h"



JpegDecoder::JpegDecoder(ConcurrentQueue<Jpeg *, 256u> *queue, QObject *parent) :
    QObject(parent), m_queue(queue)
{
}

void JpegDecoder::operate()
{
    struct jpeg_decompress_struct cinfo;
    struct error_mgr jerr;
    cinfo.err = jpeg_std_error(&jerr.mgr);
    jerr.mgr.error_exit = error_exit;
    if (setjmp(jerr.setjmp_buffer)) {
        jpeg_destroy_decompress(&cinfo);
    }
    jpeg_create_decompress(&cinfo);
    Jpeg *jpeg;
    JSAMPLE *samples;
    JSAMPROW *rows;
    forever {
        jpeg = m_queue->dequeue();
        jpeg_mem_src(&cinfo, jpeg->data, jpeg->length);
        (void) jpeg_read_header(&cinfo, TRUE);
        cinfo.dct_method = JDCT_FASTEST;
        cinfo.do_fancy_upsampling = FALSE;
        cinfo.two_pass_quantize = FALSE;
        cinfo.dither_mode = JDITHER_ORDERED;
        //cinfo.scale_num = 1;
        //cinfo.scale_denom = 8;
        cinfo.out_color_space = JCS_EXT_RGBX;
        (void) jpeg_start_decompress(&cinfo);

        samples = new JSAMPLE[cinfo.output_width * cinfo.output_height * cinfo.output_components];
        rows = new JSAMPROW[cinfo.output_height * cinfo.output_components];
        for(size_t i = 0; i < cinfo.output_height; i++) {
            rows[i] = samples + (i * cinfo.output_width * cinfo.output_components);
        }

        while (cinfo.output_scanline < cinfo.output_height) {
            jpeg_read_scanlines(&cinfo, rows + cinfo.output_scanline, 1);
        }
        (void) jpeg_finish_decompress(&cinfo);
        emit finished(samples, jpeg->x, jpeg->y, jpeg->width, jpeg->height);
        delete[] jpeg->data;
        delete jpeg;
        delete[] rows;
    }
    jpeg_destroy_decompress(&cinfo);
}
